import produce, { applyPatches } from "immer";
import equals from "shallowequal";
/**
 * Context.
 */
export class Context {
    constructor() {
        /**
         * listeners.
         */
        this.changed = [];
        /**
         * Consumer's select props.
         */
        this.selects = new Map();
        /**
         * set state.
         */
        this.setState = (state) => {
            this.state = state;
        };
        /**
         * get state.
         */
        this.getState = () => {
            return this.state;
        };
        /**
         * update state.
         */
        this.updateState = (update) => {
            // avoid produce when recursive call.
            if (this.draft) {
                update(this.draft);
                return;
            }
            // produce.
            this.state = produce(this.state, state => {
                this.draft = state;
                update(state);
                this.draft = undefined;
            }, this.patchListener);
            // calculate changes selector.
            const changed = Array.from(this.selects.entries()).reduce((changed, [select, info]) => {
                const next = select(this.state);
                if (!equals(next, info.state)) {
                    info.state = next;
                    info.version++;
                    return true;
                }
                return changed;
            }, false);
            // notify if state changes.
            if (changed) {
                this.changed.forEach(changed => changed(this.state));
            }
        };
        /**
         * apply patches.
         */
        this.applyPatches = (patches) => {
            this.updateState(state => {
                applyPatches(state, patches);
            });
        };
    }
    /**
     * subscribe state change.
     */
    subscribe(changed) {
        this.changed.push(changed);
    }
    /**
     * unsubscribe state change.
     */
    unsubscribe(changed) {
        const idx = this.changed.indexOf(changed);
        if (idx > -1) {
            this.changed.splice(idx, 1);
        }
    }
    /**
     * register select function.
     */
    register(select) {
        this.selects.set(select, {
            state: select(this.state),
            version: 1
        });
    }
    /**
     * getSelectState.
     */
    getSelectState(select) {
        return this.selects.get(select);
    }
}
//# sourceMappingURL=Context.js.map